@@ -1,10 +1,10 @@
 ; AisleRiot - agnes.scm
 ; Copyright (C) 2001, 2003 Rosanna Yuen <zana@webwynk.net>
 ;
-; This game is free software; you can redistribute it and/or modify
+; This program is free software: you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
-; the Free Software Foundation; either version 2, or (at your option)
-; any later version.
+; the Free Software Foundation, either version 3 of the License, or
+; (at your option) any later version.
 ;
 ; This program is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,9 +12,7 @@
 ; GNU General Public License for more details.
 ;
 ; You should have received a copy of the GNU General Public License
-; along with this program; if not, write to the Free Software
-; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
-; USA
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ;
 ;
 ; Andersca claims that seed 1791329065 wins
@@ -55,6 +53,7 @@
   (set! BASE-VAL (get-value (get-top-card 1)))
 
   (give-status-message)
+  (dealable-set-sensitive (dealable?))
   
   (list 7 4))
 
@@ -141,10 +140,16 @@
       (and (deal-cards-face-up 0 (list slot))
 	   (check-slot-and-deal (+ 1 slot)))))
 
+(define (do-deal-next-cards)
+  (and (dealable?)
+       (check-slot-and-deal 5)))
+
 (define (button-clicked slot-id)
   (and (= slot-id 0)
-       (not (empty-slot? 0))
-       (check-slot-and-deal 5)))
+       (do-deal-next-cards)))
+
+(define (dealable?)
+  (not (empty-slot? 0)))
 
 (define (check-dc slot f-slot just-checking?)
   (cond ((= f-slot 5)
@@ -197,6 +202,7 @@
 
 (define (game-continuable)
   (give-status-message)
+  (dealable-set-sensitive (dealable?))
   (not (game-won)))
 
 (define (game-won)
@@ -255,14 +261,14 @@
 	(#t (check-to-tableau? slot1 (+ 1 slot2)))))
 
 
-(define (dealable?)
-  (and (not (empty-slot? 0))
+(define (check-deal?)
+  (and (dealable?)
        (list 0 (_"Deal more cards"))))
 
 (define (get-hint)
   (or (check-to-foundation? 5)
       (check-to-tableau? 5 6)
-      (dealable?)
+      (check-deal?)
       (list 0 (_"Try rearranging the cards"))))
 
 (define (get-options) 
@@ -274,8 +280,8 @@
 (define (timeout) 
   #f)
 
-(set-features droppable-feature)
+(set-features droppable-feature dealable-feature)
 
 (set-lambda new-game button-pressed button-released button-clicked
 button-double-clicked game-continuable game-won get-hint get-options
-apply-options timeout droppable?)
+apply-options timeout droppable? dealable?)
