@@ -1,7 +1,7 @@
 /*
  * Copyright © 1998, 2003 Jonathan Blandford <jrb@alum.mit.edu>
  * Copyright © 2003 Callum McKenzie <callum@physics.otago.ac.nz>
- * Copyright © 2007, 2008, 2009 Christian Persch
+ * Copyright © 2007, 2008, 2009, 2010 Christian Persch
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,16 +35,13 @@
 #else
 #include <hildon/hildon-banner.h>
 #endif
-#ifdef HAVE_MAEMO_5
-#include <hildon/hildon-gtk.h>
-#include <hildon/hildon-pannable-area.h>
-#endif
 #endif /* HAVE_HILDON */
 
 #include <libgames-support/games-card-theme.h>
 #include <libgames-support/games-card-themes.h>
 #include <libgames-support/games-clock.h>
 #include <libgames-support/games-debug.h>
+#include <libgames-support/games-glib-compat.h>
 #include <libgames-support/games-stock.h>
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-sound.h>
@@ -54,12 +51,21 @@
 #include <libgames-support/games-atk-utils.h>
 #endif
 
+#ifdef HAVE_CLUTTER
+#include "ar-clutter-embed.h"
+#include "ar-style.h"
+#include "baize.h"
 #include "board.h"
+#else
+#include "board-noclutter.h"
+#endif
+
 #include "conf.h"
 #include "game.h"
 #include "stats-dialog.h"
 #include "util.h"
 #include "ar-fullscreen-button.h"
+#include "ar-game-chooser.h"
 
 #include "window.h"
 
@@ -81,8 +87,6 @@
 /* The maximum number of recent games saved */
 #define MAX_RECENT 5
 
-#define SELECT_GAME_DIALOG_MIN_HEIGHT (256)
-
 #if !GLIB_CHECK_VERSION (2, 16, 0)
 #define C_(context, string) (_(string))
 #endif
@@ -104,6 +108,10 @@
 /* define this to enable a debug menu */
 /* #undef ENABLE_DEBUG_UI */
 
+#ifdef ENABLE_DEBUG_UI
+#include "prop-editor.h"
+#endif
+
 enum
 {
   ACTION_UNDO_MOVE,
@@ -126,7 +134,14 @@ enum
 struct _AisleriotWindowPrivate
 {
   AisleriotGame *game;
+  ArStyle *board_style;
+#ifdef HAVE_CLUTTER
+  ArClutterEmbed *board;
+  ClutterActor *baize_actor;
+  ClutterActor *board_actor;
+#else
   AisleriotBoard *board;
+#endif
 
   GamesCardThemes *theme_manager;
   GamesCardTheme *theme;
@@ -139,6 +154,7 @@ struct _AisleriotWindowPrivate
 #else
   GtkStatusbar *statusbar;
   guint game_message_id;
+  guint board_message_id;
   GtkWidget *score_box;
   GtkWidget *score_label;
   GtkWidget *clock;
@@ -162,8 +178,6 @@ struct _AisleriotWindowPrivate
   GtkWidget *game_over_dialog;
   GtkWidget *game_choice_dialog;
   AisleriotStatsDialog *stats_dialog;
-  GtkListStore *game_choice_store;
-  GtkTreeSelection *game_choice_selection;
 
 #ifndef HAVE_HILDON
   GtkWidget *hint_dialog;
@@ -175,7 +189,6 @@ struct _AisleriotWindowPrivate
 
   guint load_idle_id;
 
-  guint use_pixbuf_drawing : 1;
   guint changing_game_type : 1;
   guint freecell_mode : 1;
   guint toolbar_visible : 1;
@@ -188,239 +201,11 @@ enum {
   OPTION_RADIOMENU
 };
 
-/* Game choice dialogue */
-
 enum {
-  COL_NAME,
-  COL_GAME_FILE
+  PROP_0,
+  PROP_FREECELL_MODE
 };
 
-#define SELECTED_PATH_DATA_KEY "selected-path"
-
-static void
-select_game_row_activated_cb (GtkWidget *widget,
-                              GtkTreePath *path,
-                              GtkTreeViewColumn *column,
-                              GtkDialog *dialog)
-{
-  g_assert (path != NULL);
-
-#ifdef HAVE_MAEMO_5
-  /* On maemo5 with a treeview in a pannable area, the selection
-   * handling is special. There _never_ is any row selected!
-   * Instead, we get a row-activated signal (which normally is only
-   * emitted when double-clicking a row) when the user selects a row.
-   *
-   * Since we can't use the selection, we need to store the activated
-   * row somewhere else.
-   */
-  g_object_set_data_full (G_OBJECT (dialog), SELECTED_PATH_DATA_KEY,
-                          gtk_tree_path_copy (path),
-                          (GDestroyNotify) gtk_tree_path_free);
-#endif /* HAVE_MAEMO_5 */
-
-  /* Handle a double click by faking a click on the OK button. */
-  gtk_dialog_response (dialog, GTK_RESPONSE_OK);
-}
-
-static void
-select_game_response_cb (GtkWidget *dialog,
-                         int response,
-                         AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  char *game_file = NULL;
-#ifdef HAVE_MAEMO_5
-  GtkTreePath *path;
-#endif
-
-  if (response != GTK_RESPONSE_OK)
-    goto done;
-
-#ifdef HAVE_MAEMO_5
-  /* See the comment in select_game_row_activated_cb() above. */
-  if ((path = g_object_get_data (G_OBJECT (dialog), SELECTED_PATH_DATA_KEY)) != NULL &&
-      gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->game_choice_store), &iter, path)) {
-    model = GTK_TREE_MODEL (priv->game_choice_store);
-#else
-  if (gtk_tree_selection_get_selected (priv->game_choice_selection, &model, &iter)) {
-#endif
-    gtk_tree_model_get (model, &iter,
-                        COL_GAME_FILE, &game_file,
-                        -1);
-    g_assert (game_file != NULL);
-
-    aisleriot_window_set_game (window, game_file, 0);
-
-    /* We'll store the new game in conf when it's been loaded successfully */
-
-    g_free (game_file);
-  }
-
-done:
-  gtk_widget_destroy (dialog);
-}
-
-static void
-select_game_cb (GtkAction *action,
-                AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkWidget *dialog;
-  GtkListStore *list;
-  GtkWidget *list_view;
-  GtkTreeSelection *selection;
-  GtkWidget *scrolled_window;
-  GtkTreeViewColumn *column;
-  GtkCellRenderer *renderer;
-  GtkWidget *hbox, *content_area;
-  GtkTreeIter current_iter;
-  gboolean current_iter_set = FALSE;
-  const char *current_game_file;
-  const char *games_dir;
-  GDir *dir;
-
-  if (priv->game_choice_dialog) {
-    gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
-    return;
-  }
-
-  priv->game_choice_store = list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-
-#ifdef HAVE_MAEMO_5
-  list_view = hildon_gtk_tree_view_new_with_model (HILDON_UI_MODE_NORMAL, GTK_TREE_MODEL (list));
-#else
-  list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list));
-#endif
-  g_object_unref (list);
-
-  current_game_file = aisleriot_game_get_game_file (priv->game);
-
-  games_dir = games_runtime_get_directory (GAMES_RUNTIME_GAME_GAMES_DIRECTORY);
-
-  dir = g_dir_open (games_dir, 0, NULL);
-  if (dir != NULL) {
-    const char *game_file;
-
-    while ((game_file = g_dir_read_name (dir)) != NULL) {
-      char *game_name;
-      GtkTreeIter iter;
-
-      if (!g_str_has_suffix (game_file, ".scm") ||
-          strcmp (game_file, "sol.scm") == 0)
-        continue;
-
-      game_name = games_filename_to_display_name (game_file);
-
-      gtk_list_store_insert_with_values (GTK_LIST_STORE (list), &iter,
-                                         -1,
-                                         COL_NAME, game_name,
-                                         COL_GAME_FILE, game_file,
-                                         -1);
-
-      if (current_game_file &&
-          strcmp (current_game_file, game_file) == 0) {
-        current_iter = iter;
-        current_iter_set = TRUE;
-      }
-
-      g_free (game_name);
-    }
-
-    g_dir_close (dir);
-  }
-
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list),
-                                        0, GTK_SORT_ASCENDING);
-
-  dialog = gtk_dialog_new_with_buttons (_("Select Game"),
-                                        GTK_WINDOW (window),
-                                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        _("_Select"), GTK_RESPONSE_OK,
-                                        NULL);
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-  content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-  gtk_box_set_spacing (GTK_BOX (content_area), 2);
-
-  hbox = gtk_hbox_new (FALSE, 12);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-  gtk_box_pack_start (GTK_BOX (content_area), hbox, TRUE, TRUE, 0);
-  g_signal_connect (list_view, "row-activated",
-                    G_CALLBACK (select_game_row_activated_cb), dialog);
-
-  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list_view), FALSE);
-
-  renderer = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes (NULL,
-                                                     renderer,
-                                                     "text", COL_NAME, NULL);
-
-  gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), column);
-
-  priv->game_choice_selection = selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
-  gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
-
-#ifdef HAVE_MAEMO_5
-  scrolled_window = hildon_pannable_area_new ();
-#else
-  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
-                                       GTK_SHADOW_IN);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-#endif /* HAVE_MAEMO_5 */
-
-  gtk_container_add (GTK_CONTAINER (scrolled_window), list_view);
-
-  gtk_box_pack_end (GTK_BOX (hbox), scrolled_window, TRUE, TRUE, 0);
-
-  /* Set initial focus to the list view. Otherwise, if you press Up/Down key,
-   * the selection jumps to the first entry in the list as the view gains
-   * focus.
-   */
-  gtk_widget_grab_focus (list_view);
-
-  gtk_widget_show_all (hbox);
-
-  g_signal_connect (dialog, "response",
-                    G_CALLBACK (select_game_response_cb), window);
-
-  priv->game_choice_dialog = dialog;
-  g_signal_connect (dialog, "destroy",
-                    G_CALLBACK (gtk_widget_destroyed), &priv->game_choice_dialog);
-
-  gtk_widget_set_size_request (scrolled_window, -1, SELECT_GAME_DIALOG_MIN_HEIGHT);
-  gtk_window_present (GTK_WINDOW (dialog));
-
-  /* Select the row corresponding to the currently loaded game,
-   * and scroll to it.
-   */
-  if (current_iter_set) {
-    GtkTreePath *path;
-
-    gtk_tree_selection_select_iter (selection, &current_iter);
-
-    /* Scroll view to the current item */
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (list), &current_iter);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_view), path, NULL,
-                                  TRUE,
-				  0.5, 0.0);
-    gtk_tree_path_free (path);
-  }
-}
-
 /* Game Over dialogue */
 
 enum {
@@ -592,7 +377,7 @@ new_game_cb (GtkAction *action,
   aisleriot_game_new_game (priv->game, NULL);
 
   gtk_widget_grab_focus (GTK_WIDGET (priv->board));
-};
+}
 
 static void
 undo_cb (GtkAction *action,
@@ -601,7 +386,11 @@ undo_cb (GtkAction *action,
   AisleriotWindowPrivate *priv = window->priv;
 
   /* If a move is in progress, cancel it before changing the game! */
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
   aisleriot_board_abort_move (priv->board);
+#endif
 
   aisleriot_game_undo_move (priv->game);
 }
@@ -612,8 +401,11 @@ redo_cb (GtkAction *action,
 {
   AisleriotWindowPrivate *priv = window->priv;
 
-  /* If a move is in progress, cancel it before changing the game! */
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
   aisleriot_board_abort_move (priv->board);
+#endif
 
   aisleriot_game_redo_move (priv->game);
 }
@@ -691,7 +483,7 @@ help_about_cb (GtkAction *action,
 			     "AisleRiot is a part of GNOME Games."),
 #endif
                          "copyright", "Copyright © 1998-2006 Jonathan Blandford\n"
-                                      "Copyright © 2007, 2008, 2009 Christian Persch",
+                                      "Copyright © 2007, 2008, 2009, 2010 Christian Persch",
                          "license", licence,
                          "authors", authors,
                          "artists", artists,
@@ -717,6 +509,24 @@ restart_game (GtkAction *action,
   aisleriot_game_restart_game (priv->game);
 };
 
+static void
+select_game_cb (GtkAction *action,
+                AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  if (priv->game_choice_dialog) {
+    gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
+    return;
+  }
+
+  priv->game_choice_dialog = ar_game_chooser_new (window);
+  g_signal_connect (priv->game_choice_dialog, "destroy",
+                    G_CALLBACK (gtk_widget_destroyed), &priv->game_choice_dialog);
+
+  gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
+}
+
 static void
 help_general_cb (GtkAction *action,
                  AisleriotWindow *window)
@@ -816,12 +626,7 @@ static void
 delayed_move_to_next_screen_cb (GtkAction *action,
                                 GtkWidget *widget)
 {
-#if GLIB_CHECK_VERSION (2, 14, 0)
   g_timeout_add_seconds (10, (GSourceFunc) delayed_move_to_next_screen_timeout_cb, widget);
-#else
-  g_timeout_add (10 * 1000, (GSourceFunc) delayed_move_to_next_screen_timeout_cb, widget);
-#endif /* GLIB 2.14.0 */
-
 }
 
 static void
@@ -1049,14 +854,22 @@ debug_choose_seed_cb (GtkAction *action,
 }
 
 static void
-debug_pixbuf_drawing_cb (GtkToggleAction *action,
-                         AisleriotWindow *window)
+debug_tweak_cb (GtkAction *action,
+                AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
-  gboolean active;
+  GObject *object;
+  GType type;
+  GtkWidget *prop_editor;
 
-  active = gtk_toggle_action_get_active (action);
-  aisleriot_board_set_pixbuf_drawing (priv->board, active);
+  object = G_OBJECT (priv->board_style);
+  type = G_OBJECT_TYPE (object);
+
+  g_assert (object != NULL);
+
+  prop_editor = create_prop_editor (object, type);
+  gtk_window_set_transient_for (GTK_WINDOW (prop_editor), GTK_WINDOW (window));
+  gtk_window_present (GTK_WINDOW (prop_editor));
 }
 
 #endif /* ENABLE_DEBUG_UI */
@@ -1188,7 +1001,7 @@ clickmove_toggle_cb (GtkToggleAction *action,
   click_to_move = gtk_toggle_action_get_active (action);
 
   aisleriot_game_set_click_to_move (priv->game, click_to_move);
-  aisleriot_board_set_click_to_move (priv->board, click_to_move);
+  ar_style_set_click_to_move (priv->board_style, click_to_move);
   
   games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_CLICK_TO_MOVE), click_to_move);
 }
@@ -1221,7 +1034,7 @@ animations_toggle_cb (GtkToggleAction *action,
 
   enabled = gtk_toggle_action_get_active (action);
 
-  aisleriot_board_set_animation_mode (priv->board, enabled);
+  ar_style_set_enable_animations (priv->board_style, enabled);
   
   games_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), enabled);
 }
@@ -1605,7 +1418,7 @@ aisleriot_window_take_card_theme (AisleriotWindow *window,
   }
 #endif /* GTK+ 2.10.0 */
 
-  aisleriot_board_set_card_theme (priv->board, theme);
+  ar_style_set_card_theme (priv->board_style, theme);
 }    
 
 static void
@@ -1925,7 +1738,10 @@ game_type_changed_cb (AisleriotGame *game,
   g_object_set (priv->action[ACTION_HELP_GAME], "label", game_name, NULL);
   g_object_set (priv->action[ACTION_OPTIONS_MENU], "label", game_name, NULL);
 
-  gtk_window_set_title (GTK_WINDOW (window), game_name);
+  /* In freecell mode, we've already set the title to something different */
+  if (!priv->freecell_mode) {
+    gtk_window_set_title (GTK_WINDOW (window), game_name);
+  }
 
   g_free (game_name);
 
@@ -1950,6 +1766,7 @@ game_type_changed_cb (AisleriotGame *game,
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
   gtk_statusbar_pop (priv->statusbar, priv->game_message_id);
+  gtk_statusbar_pop (priv->statusbar, priv->board_message_id);
 
   show_scores = (features & FEATURE_SCORE_HIDDEN) == 0;
   g_object_set (priv->score_box, "visible", show_scores, NULL);
@@ -1976,6 +1793,7 @@ game_new_cb (AisleriotGame *game,
   games_clock_reset (GAMES_CLOCK (priv->clock));
 
   gtk_statusbar_pop (priv->statusbar, priv->game_message_id);
+  gtk_statusbar_pop (priv->statusbar, priv->board_message_id);
 #endif /* HAVE_HILDON */
 }
 
@@ -2043,6 +1861,13 @@ game_exception_response_cb (GtkWidget *dialog,
       "--appname", "aisleriot",
       "--pid", pidstr,
       "--include", (const char *) error_file,
+#if GLIB_CHECK_VERSION (2, 20, 0)
+      /* This option was added to bug-buddy 2.25.x (bug 540150). We use the
+       * 2.26 glib version as a proxy to detect this, since there's no good
+       * other way.
+       */
+      "--unlink-tempfile",
+#endif
       NULL
     };
 
@@ -2117,7 +1942,7 @@ game_exception_cb (AisleriotGame *game,
   gtk_widget_show (dialog);
 }
 
-#if defined(HAVE_CLUTTER) || (defined(HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0))
+#if defined(HAVE_CLUTTER) || (defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0))
 
 static void
 settings_changed_cb (GtkSettings *settings,
@@ -2143,14 +1968,14 @@ settings_changed_cb (GtkSettings *settings,
   }
 #endif /* HAVE_CLUTTER */
 
-#if defined(HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0)
+#if defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   if (name == NULL || strcmp (name, "gtk-enable-event-sounds") == 0) {
     g_object_get (settings, "gtk-enable-event-sounds", &enabled, NULL);
 
     action = gtk_action_group_get_action (priv->action_group, "Sound");
     gtk_action_set_visible (action, enabled);
   }
-#endif /* HAVE_CANBERRA_GTK && GTK >= 2.14 */
+#endif /* ENABLE_SOUND && GTK >= 2.14 */
 }
 
 static void
@@ -2182,13 +2007,92 @@ screen_changed_cb (GtkWidget *widget,
   g_signal_connect (settings, "notify::gtk-enable-animations",
                     G_CALLBACK (settings_changed_cb), window);
 #endif
-#if defined (HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0)
+#if defined (ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0)
   g_signal_connect (settings, "notify::gtk-enable-event-sounds",
                     G_CALLBACK (settings_changed_cb), window);
 #endif
 }
 
-#endif /* HAVE_CLUTTER || HAVE_CANBERRA_GTK */
+#endif /* HAVE_CLUTTER || ENABLE_SOUND && GTK+ >= 2.14.0 */
+
+/*
+ * aisleriot_window_set_freecell_mode:
+ * @window:
+ *
+ * Sets @window to FreeCell mode. In FreeCell mode,
+ * the window is using the FreeCell variation, and doesn't allow
+ * changing the game type.
+ */
+static void
+aisleriot_window_set_freecell_mode (AisleriotWindow *window,
+                                    gboolean freecell_mode)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+  GtkAction *action;
+
+  priv->freecell_mode = freecell_mode != FALSE;
+
+  if (freecell_mode) {
+    /* Inhibit game changing */
+    action = gtk_action_group_get_action (priv->action_group, "Select");
+    gtk_action_set_visible (action, FALSE);
+    action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
+    gtk_action_set_visible (action, FALSE);
+
+    gtk_window_set_title (GTK_WINDOW (window), _("Freecell Solitaire"));
+  } else {
+    gtk_window_set_title (GTK_WINDOW (window), _("AisleRiot"));
+  }
+}
+
+#ifndef HAVE_HILDON
+
+static void
+board_status_message_cb (AisleriotBoard *board,
+                         const char *status_message,
+                         AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  gtk_statusbar_pop (priv->statusbar, priv->board_message_id);
+
+  if (status_message != NULL) {
+    gtk_statusbar_push (priv->statusbar, priv->board_message_id, status_message);
+  }
+}
+
+#endif /* !HAVE_HILDON */
+
+#ifdef HAVE_CLUTTER
+
+static void
+board_cursor_cb (AisleriotBoard *board,
+                 int cursor_type,
+                 ArClutterEmbed *embed)
+{
+  ar_clutter_embed_set_cursor (embed, (ArCursorType) cursor_type);
+}
+
+static void
+board_error_bell_cb (AisleriotBoard *board,
+                     ArClutterEmbed *embed)
+{
+#if GTK_CHECK_VERSION (2, 12, 0) || (defined (HAVE_HILDON) && !defined(HAVE_MAEMO_3))
+  gtk_widget_error_bell (GTK_WIDGET (embed));
+#endif
+}
+
+static void
+embed_size_allocate_cb (ArClutterEmbed *embed,
+                        GtkAllocation *allocation,
+                        AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  clutter_actor_set_size (priv->board_actor, allocation->width, allocation->height);
+}
+
+#endif /* HAVE_CLUTTER */
 
 /* Class implementation */
 
@@ -2375,6 +2279,8 @@ aisleriot_window_init (AisleriotWindow *window)
     { "DebugGamePrev", GTK_STOCK_GO_BACK, NULL, NULL, NULL,
       G_CALLBACK (debug_game_prev) },
 #endif /* !HAVE_HILDON */
+    { "DebugTweakStyle", NULL, "_Tweak Style", NULL, NULL,
+      G_CALLBACK (debug_tweak_cb) },
 #endif /* ENABLE_DEBUG_UI */
 
     /* Accel actions */
@@ -2416,11 +2322,6 @@ aisleriot_window_init (AisleriotWindow *window)
       G_CALLBACK (animations_toggle_cb),
       FALSE /* not active by default */ },
 #endif /* HAVE_CLUTTER */
-#ifdef ENABLE_DEBUG_UI
-    { "DebugPixbufDrawing", NULL, "_Pixbuf drawing", NULL, NULL,
-      G_CALLBACK (debug_pixbuf_drawing_cb),
-      FALSE },
-#endif /* ENABLE_DEBUG_UI */
   };
 
   static const char names[][16] = {
@@ -2480,7 +2381,7 @@ aisleriot_window_init (AisleriotWindow *window)
 #ifdef ENABLE_DEBUG_UI
         "<menu action='DebugMenu'>"
           "<menuitem action='DebugChooseSeed'/>"
-          "<menuitem action='DebugPixbufDrawing'/>"
+          "<menuitem action='DebugTweakStyle'/>"
         "</menu>"
 #endif /* ENABLE_DEBUG_UI */
         "<menuitem action='CloseWindow'/>"
@@ -2531,7 +2432,7 @@ aisleriot_window_init (AisleriotWindow *window)
         "</menu>"
 #ifdef ENABLE_DEBUG_UI
         "<menu action='DebugMenu'>"
-          "<menuitem action='DebugPixbufDrawing'/>"
+          "<menuitem action='DebugTweakStyle'/>"
           "<separator/>"
           "<menuitem action='DebugChooseSeed'/>"
           "<menuitem action='DebugMoveNextScreen'/>"
@@ -2584,11 +2485,11 @@ aisleriot_window_init (AisleriotWindow *window)
   GamesCardTheme *theme;
   guint i;
 #ifndef HAVE_HILDON
-  const char *env;
   GtkStatusbar *statusbar;
-  GtkWidget *statusbar_hbox, *statusbar_label, *label, *time_box;
-  GtkContainer *statusbar_frame;
-  GList *list;
+  GtkWidget *statusbar_hbox, *label, *time_box;
+#endif
+#ifdef HAVE_CLUTTER
+  ClutterContainer *stage;
 #endif
 
   g_assert (G_N_ELEMENTS (names) == LAST_ACTION);
@@ -2599,26 +2500,38 @@ aisleriot_window_init (AisleriotWindow *window)
 
   priv->game = aisleriot_game_new ();
 
-#ifdef HAVE_HILDON
-  priv->use_pixbuf_drawing = FALSE;
-#else
-  /* Default to pixbuf drawing */
-  env = g_getenv ("AISLERIOT_PIXBUF_DRAWING");
-  priv->use_pixbuf_drawing = env == NULL || g_ascii_strtoull (env, NULL, 10) != 0;
-#endif /* HAVE_HILDON */
-
-#ifdef GNOME_ENABLE_DEBUG
-  if (priv->use_pixbuf_drawing)
-    g_print ("Using pixbuf drawing method\n");
-  else
-    g_print ("Using pixmap drawing method\n");
-#endif /* GNOME_ENABLE_DEBUG */
-
   priv->theme_manager = games_card_themes_new ();
 
-  priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->game));
+  priv->board_style = ar_style_new ();
 
-  aisleriot_board_set_pixbuf_drawing (priv->board, priv->use_pixbuf_drawing);
+#ifdef HAVE_CLUTTER
+  priv->board = ar_clutter_embed_new (priv->board_style);
+
+  priv->baize_actor = aisleriot_baize_new ();
+
+  stage = CLUTTER_CONTAINER (gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (priv->board)));
+  clutter_container_add (stage, priv->baize_actor, NULL);
+  /* FIXMEchpe: how to ensure this is ALWAYS the lowest actor? */
+  clutter_actor_lower_bottom (priv->baize_actor);
+
+  priv->board_actor = aisleriot_board_new (priv->board_style, priv->game);
+  clutter_container_add (stage, priv->board_actor, NULL);
+
+  /* FIXMEchpe */
+  clutter_stage_set_key_focus (CLUTTER_STAGE (stage), priv->board_actor);
+
+  g_signal_connect_after (priv->board, "size-allocate",
+                          G_CALLBACK (embed_size_allocate_cb), window);
+
+  g_signal_connect (priv->board_actor, "request-cursor",
+                    G_CALLBACK (board_cursor_cb), priv->board);
+  g_signal_connect (priv->board_actor, "error-bell",
+                    G_CALLBACK (board_error_bell_cb), priv->board);
+
+  /* FIXMEchpe: unref baize & board_actor here? */
+#else
+  priv->board = AISLERIOT_BOARD (aisleriot_board_new (priv->board_style, priv->game));
+#endif /* HAVE_CLUTTER */
 
   theme_name = games_conf_get_string (NULL, aisleriot_conf_get_key (CONF_THEME), NULL);
   theme = games_card_themes_get_theme_by_name (priv->theme_manager, theme_name);
@@ -2668,12 +2581,31 @@ aisleriot_window_init (AisleriotWindow *window)
   games_stock_prepare_for_statusbar_tooltips (priv->ui_manager,
                                               GTK_WIDGET (priv->statusbar));
 
+  priv->game_message_id = gtk_statusbar_get_context_id (priv->statusbar, "board-message");
+
+#ifdef HAVE_CLUTTER
+  g_signal_connect (priv->board_actor, "status-message",
+                    G_CALLBACK (board_status_message_cb), window);
+#else
+  g_signal_connect (priv->board, "status-message",
+                    G_CALLBACK (board_status_message_cb), window);
+#endif
+
 #if GTK_CHECK_VERSION (2, 11, 0)
   gtk_statusbar_set_has_resize_grip (priv->statusbar, TRUE);
 #else
   gtk_statusbar_set_has_resize_grip (priv->statusbar, FALSE);
 #endif
 
+#if GTK_CHECK_VERSION (2, 19, 1)
+  statusbar_hbox = gtk_statusbar_get_message_area (statusbar);
+  gtk_box_set_spacing (GTK_BOX (statusbar_hbox), 24);
+#else
+{
+  GtkWidget *statusbar_label;
+  GtkContainer *statusbar_frame;
+  GList *list;
+
   /* Widget surgery: move the statusbar's label into a hbox
    * which we put in the statusbar's frame instead.
    */
@@ -2688,6 +2620,8 @@ aisleriot_window_init (AisleriotWindow *window)
   g_object_unref (statusbar_label);
   gtk_container_add (statusbar_frame, statusbar_hbox);
   gtk_widget_show (statusbar_hbox);
+}
+#endif /* GTK+ >= 2.19.1 */
 
   /* Score */
   priv->score_box = gtk_hbox_new (12, FALSE);
@@ -2821,12 +2755,12 @@ aisleriot_window_init (AisleriotWindow *window)
 
 #endif /* HAVE_CLUTTER */
 
-#if defined(HAVE_CLUTTER) || (defined(HAVE_CANBERRA_GTK) && GTK_CHECK_VERSION (2, 14, 0))
+#if defined(HAVE_CLUTTER) || (defined(ENABLE_SOUND) && GTK_CHECK_VERSION (2, 14, 0))
   /* Set the action visibility and listen for animation and sound mode changes */
   screen_changed_cb (GTK_WIDGET (window), NULL, window);
   g_signal_connect (window, "screen-changed",
                     G_CALLBACK (screen_changed_cb), window);
-#endif /* HAVE_CLUTTER || HAVE_CANBERRA_GTK */
+#endif /* HAVE_CLUTTER || ENABLE_SOUND && GTK+ >= 2.14.0 */
 
   /* Now set up the widgets */
   main_vbox = gtk_vbox_new (FALSE, 0);
@@ -2900,11 +2834,6 @@ aisleriot_window_init (AisleriotWindow *window)
   g_signal_connect (window, "notify::is-topmost",
                     G_CALLBACK (sync_window_topmost_cb), NULL);
 #endif
-
-#ifdef ENABLE_DEBUG_UI
-  action = gtk_action_group_get_action (priv->action_group, "DebugPixbufDrawing");
-  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), priv->use_pixbuf_drawing);
-#endif /* ENABLE_DEBUG_UI */
 }
 
 static void
@@ -2959,6 +2888,10 @@ aisleriot_window_finalize (GObject *object)
   AisleriotWindow *window = AISLERIOT_WINDOW (object);
   AisleriotWindowPrivate *priv = window->priv;
 
+#ifdef HAVE_CLUTTER
+  g_object_unref (priv->board_style);
+#endif /* HAVE_CLUTTER */
+
   if (priv->theme) {
     g_object_unref (priv->theme);
   }
@@ -2973,6 +2906,23 @@ aisleriot_window_finalize (GObject *object)
   G_OBJECT_CLASS (aisleriot_window_parent_class)->finalize (object);
 }
 
+static void
+aisleriot_window_set_property (GObject      *object,
+                               guint         property_id,
+                               const GValue *value,
+                               GParamSpec   *pspec)
+{
+  AisleriotWindow *window = AISLERIOT_WINDOW (object);
+
+  switch (property_id) {
+    case PROP_FREECELL_MODE:
+      aisleriot_window_set_freecell_mode (window, g_value_get_boolean (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+  }
+}
 static void
 aisleriot_window_class_init (AisleriotWindowClass *klass)
 {
@@ -2981,6 +2931,7 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
 
   gobject_class->dispose = aisleriot_window_dispose;
   gobject_class->finalize = aisleriot_window_finalize;
+  gobject_class->set_property = aisleriot_window_set_property;
 
   widget_class->window_state_event = aisleriot_window_state_event;
 #if GTK_CHECK_VERSION (2, 10, 0)
@@ -2988,6 +2939,20 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
 #endif
 
   g_type_class_add_private (gobject_class, sizeof (AisleriotWindowPrivate));
+
+  /**
+   * AisleriotWindow:freecell-mode:
+   *
+   * Whether the window is in freecell mode.
+   */
+  g_object_class_install_property
+    (gobject_class,
+     PROP_FREECELL_MODE,
+     g_param_spec_boolean ("freecell-mode", NULL, NULL,
+                           FALSE,
+                           G_PARAM_WRITABLE |
+                           G_PARAM_CONSTRUCT_ONLY |
+                           G_PARAM_STATIC_STRINGS));
 }
 
 /* public API */
@@ -2998,36 +2963,13 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
  * Returns: a new #AisleriotWindow
  */
 GtkWidget *
-aisleriot_window_new (void)
+aisleriot_window_new (gboolean freecell_mode)
 {
   return g_object_new (AISLERIOT_TYPE_WINDOW,
-                       "title", _("AisleRiot"),
+                       "freecell-mode", freecell_mode,
                        NULL);
 }
 
-/**
- * aisleriot_window_set_freecell_mode:
- * @window:
- *
- * Sets @window to FreeCell mode. In FreeCell mode,
- * the window is using the FreeCell variation, and doesn't allow
- * changing the game type.
- */
-void
-aisleriot_window_set_freecell_mode (AisleriotWindow *window)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkAction *action;
-
-  priv->freecell_mode = TRUE;
-
-  /* Inhibit game changing */
-  action = gtk_action_group_get_action (priv->action_group, "Select");
-  gtk_action_set_visible (action, FALSE);
-  action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
-  gtk_action_set_visible (action, FALSE);
-}
-
 typedef struct {
   AisleriotWindow *window;
   char *game_file;
@@ -3106,11 +3048,7 @@ free_load_idle_data (LoadIdleData *data)
   data->window->priv->load_idle_id = 0;
 
   g_free (data->game_file);
-#if GLIB_CHECK_VERSION (2, 10, 0)
   g_slice_free (LoadIdleData, data);
-#else
-  g_free (data);
-#endif
 }
 
 /**
@@ -3136,12 +3074,7 @@ aisleriot_window_set_game (AisleriotWindow *window,
     g_source_remove (priv->load_idle_id);
   }
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
   data = g_slice_new (LoadIdleData);
-#else
-  data = g_new (LoadIdleData, 1);
-#endif
-
   data->window = window;
   data->game_file = g_strdup (game_file);
   data->seed = seed;
