@@ -1,7 +1,7 @@
-/* 
+/*
  *   Gnome Nibbles: Gnome Worm Game
  *   Written by Sean MacIsaac <sjm@acm.org>, Ian Peters <itp@gnu.org>
- * 
+ *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -23,16 +23,19 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
+#include <clutter/clutter.h>
 
 #define WARP     8
 
 typedef struct {
   gint x, y;
   gint wx, wy;
+  ClutterActor *actor;
 } GnibblesWarp;
 
 GnibblesWarp *gnibbles_warp_new (gint t_x, gint t_y, gint t_wx, gint t_wy);
 
 void gnibbles_warp_draw_buffer (GnibblesWarp * warp);
+void gnibbles_warp_draw (GnibblesWarp *warp);
 
 #endif
