@@ -1,20 +1,20 @@
 /*
-   Copyright © 2004 Callum McKenzie
-   Copyright © 2007, 2008, 2009 Christian Persch
+  Copyright © 2004 Callum McKenzie
+  Copyright © 2007, 2008, 2009 Christian Persch
 
-   This library is free software; you can redistribute it and'or modify
-   it under the terms of the GNU Library General Public License as published 
-   by the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+  This library is free software: you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public License
-   along with this library; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+  You should have received a copy of the GNU Lesser General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /* Authors:   Callum McKenzie <callum@physics.otago.ac.nz> */
 
@@ -130,7 +130,7 @@ static char *
 theme_filename_and_type_from_name (const char *theme_name,
                                    GType *type)
 {
-  const char *colon;
+  const char *colon, *filename, *dot;
 
   g_return_val_if_fail (type != NULL, NULL);
 
@@ -146,46 +146,45 @@ theme_filename_and_type_from_name (const char *theme_name,
   if (*type == G_TYPE_INVALID)
     return NULL;
 
+  /* Get the filename from the theme name */
   if (colon) {
-    const char *filename, *dot;
-
-    /* Get the filename from the theme name */
     filename = colon + 1;
-    dot = strrchr (filename, '.');
-    if (filename == dot || !filename[0])
-      return NULL;
-
-    if (dot == NULL) {
-      /* No dot? Try appending the default, for compatibility with old settings */
-#if defined(ENABLE_CARD_THEME_FORMAT_FIXED)
-      if (*type == GAMES_TYPE_CARD_THEME_FIXED) {
-        return g_strconcat (filename, ".card-theme", NULL);
-      }
-#elif defined(ENABLE_CARD_THEME_FORMAT_SVG)
-      if (*type == GAMES_TYPE_CARD_THEME_SVG) {
-        return g_strconcat (filename, ".svg", NULL);
-      }
-#endif
-    } else {
-#if defined(HAVE_GNOME) && defined(ENABLE_CARD_THEME_FORMAT_SVG)
-      if (*type == GAMES_TYPE_CARD_THEME_SVG &&
-          g_str_has_suffix (filename, ".png")) {
-        char *base_name, *retval;
-
-        /* Very old version; replace .png with .svg */
-        base_name = g_strndup (filename, dot - filename);
-        retval = g_strconcat (base_name, ".svg", NULL);
-        g_free (base_name);
-
-        return retval;
-      }
-#endif /* HAVE_GNOME && ENABLE_CARD_THEME_FORMAT_SVG */
-    }
-
-    return g_strdup (filename);
+  } else {
+    filename = theme_name;
   }
 
-  return g_strdup (theme_name);
+  dot = strrchr (filename, '.');
+  if (filename == dot || !filename[0])
+    return NULL;
+
+  if (dot == NULL) {
+    /* No dot? Try appending the default, for compatibility with old settings */
+#if defined(ENABLE_CARD_THEME_FORMAT_FIXED)
+    if (*type == GAMES_TYPE_CARD_THEME_FIXED) {
+      return g_strconcat (filename, ".card-theme", NULL);
+    }
+#elif defined(ENABLE_CARD_THEME_FORMAT_SVG)
+    if (*type == GAMES_TYPE_CARD_THEME_SVG) {
+      return g_strconcat (filename, ".svg", NULL);
+    }
+#endif
+  } else {
+#if defined(HAVE_GNOME) && defined(ENABLE_CARD_THEME_FORMAT_SVG)
+    if (*type == GAMES_TYPE_CARD_THEME_SVG &&
+        g_str_has_suffix (filename, ".png")) {
+      char *base_name, *retval;
+
+      /* Very old version; replace .png with .svg */
+      base_name = g_strndup (filename, dot - filename);
+      retval = g_strconcat (base_name, ".svg", NULL);
+      g_free (base_name);
+
+      return retval;
+    }
+#endif /* HAVE_GNOME && ENABLE_CARD_THEME_FORMAT_SVG */
+  }
+
+  return g_strdup (filename);
 }
 
 static gboolean
